/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CONTROL_H
#define CONTROL_H

#include <sigc++/trackable.h>


#define CONTROL Control::getInstance()

// Forward declaration
class SelectionMenu;
class MainMenu;
class ModuleManager;
class MediaPlayer;
class View;
class Menu;

/**
 * @class Control
 * @brief Implements the logic behind interactions between the view and 
 *        the menus.
 *
 * This class is a singleton.  init() must have been called before it can be
 * used.
 */

class Control: public sigc::trackable {
  // Associations
  // Attributes
  private:
    SelectionMenu *   _rootMenu;
    View *            _view;
    MediaPlayer *     _mediaPlayer;
    ModuleManager *   _modules;
    
    
    Control();  // Singleton design pattern
    ~Control();
    
    Menu * getSelectedMenu (Menu * root);  //recursive getSelected
     
    /**
     * @brief Sets the root of the menu tree.
     * @param root The root menu.
     */
    void setRootMenu ( MainMenu * root );
    
    /**
     * @brief Sets the view pointer inside the controler.
     * @param view The view
     */
    void setView ( View * view );
    
  // Operations
  public:
    
    /**
     * @brief Returns a pointer to the Control singleton object.
     * @return A pointer to the Control singleton object.
     */
    static Control * getInstance (  );
    
    /**
     * @brief Returns the root of the menu tree.
     * @return The root of the menu tree.
     */
    Menu * getRootMenu (  );
    
    /**
     * @brief Returns a pointer to the view, which is the window.
     * @return A pointer to the view.
     */
    View * getView (  );
    
    /**
     * @brief Returns a pointer to the media player, which plays the content.
     * @return A pointer to the media player.
     */
    MediaPlayer * getMediaPlayer (  );
    
    /**
     * @brief Handles the keys pressed by the user.
     * @param key The X key number.
     * @return True if the key was catch, false if the key should be sent to the 
     *         OS
     *
     */
    bool keypressed ( int key );
    
    /**
     * @brief Returns the current menu.
     * 
     * A selected menu is a menu which isSelected(), there can only be one in the
     * menu tree.  If the selected menu has sub menus, they should be presented
     * to the user so he can make a choice.
     * @return The current menu.
     */
    Menu * getSelectedMenu (  );
    
    /**
     * @brief Initialise the modules and menus.
     */
    void init ( int argc, char * argv[] );
    
    /**
     * @brief Uninitialise the modules and menus.
     */
    void uninit (  );
    
};

#endif
