/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef MODULE_H
#define MODULE_H

#include <sigc++/trackable.h>
#include <ltdl.h>               // lt_dlhandle
#include <glib.h>

// Forward declaration
class Menu;
class ConfigWindow;

/**
 * @class Module
 * @brief Represents a module.
 *
 * This abstract class is aimed to be implemented by modules as an API to the 
 * module.  Upon loading a module, getMenu() will be called to attach the module
 * in the menu tree.  
 */

class Module: public sigc::trackable {
  // Associations
  // Attributes
  private:
    lt_dlhandle _handle;
  // Operations
  public:
    Module();
    ~Module();
    
    /**
     * @brief Returns the name of the module.
     * @return The name of the module.
     */
    virtual const gchar * getName (  ) = 0;
    
    /**
     * A setting name is the name of the settings folder under /apps/sofa/
     * @brief Returns the Setting Name of the module.
     * @return The setting name of the module.
     */
    virtual const gchar *  getSettingName (  ) = 0;
    
    /**
     * @brief Returns the version string for the module.
     * @return The version of the module.
     */
    virtual const gchar *  getVersion (  ) = 0;
    
    /**
     * @brief Returns author information of the module.
     *
     * If more than one author exist, they should separate their name
     * by a semi-colomn.
     * @return The author(s) of the module.
     */
    virtual const gchar *  getAuthor (  ) = 0;
    
    /**
     * @brief Returns the description text for the module.
     *
     * Should be around less than 30 words.
     * @return The description of the module.
     */
    virtual const gchar *  getDescription (  ) = 0;
    
    /**
     * @brief Unloads the internaly used objected of the module.
     */
    virtual void unload (  ) = 0;
    
    /**
     * @brief Returns the root menu of the module.
     * @return The root menu of the module.
     */
    virtual Menu * getMenu (  ) = 0;
    
    /**
     * @brief Returns the configuration window
     * @return The configuration window of the module.
     */
    virtual ConfigWindow * getConfigWindow (  ) = 0;
    
    
    /**
     * @brief Returns handle to the module as returned by libtool.
     * @return The handle on the module.
     */
    lt_dlhandle getHandle (  );
    
    /**
     * @brief Sets the handle received from libtool.
     * @param[in] handle The position in the list.
     */
    void setHandle ( lt_dlhandle handle );
};

#endif
