/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef MODULEMANAGER_H
#define MODULEMANAGER_H

#include "module.h"

#include <list>
#include <string>

/**
 * @class ModuleManager
 * @brief In charge of loading and unloading modules.
 *
 * This class meant to be used inside Control or in sofa-config.  init() must 
 * have been called before it can be used.  This object will load the DSOs and
 * query them for their Module object, which is kept in a list.
 */

class ModuleManager {
  // Associations
  // Attributes
  private:
    std::list<Module*> _modules;
    std::string        _path;
    
  // Operations
  public:
    ModuleManager();
    
    /**
     * @brief Loads modules. 
     * @param activeOnly If true, only modules that have been marked as active 
     * will be loaded, if false, all modules available in the path will be loaded.
     */
    void loadModules ( bool activeOnly );
    
    /**
     * @brief Unloads modules. 
     *
     * Each modules that have been loaded (are in the list) will be unloaded.
     */
    void unloadModules (  );
    
    /**
     * @brief Gives access to modules.
     * @param pos The index in the list of modules.
     * @return A pointer on the requested Module object.
     */
    Module * getModule ( int pos );
    
    /**
     * @brief Count the number of loaded modules.
     * @return The number of mudules in the list.
     */
    int count (  );
  private:
    /**
     * @brief Load one module.
     * @param[in] filename The filename of the module.
     * @param[out] mod The Module object from the module.
     * @return True if suceeded, false otherwise.
     */
    bool loadModule (  std::string filename, Module * mod );
    
    /**
     * @brief Unload one module.
     * @param The Module object of which the module should be unloaded.
     * @return True if suceeded, false otherwise.
     */
    bool unloadModule(Module * mod);
    
  
};

#endif
