/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "rbmediaplayer.h"
#include "../../clip.h"
#include "../../fileutils.h"

#include <dbus/dbus-glib.h>
#define DBUS_API_SUBJECT_TO_CHANGE 0 // Because dbus < 1.0
#include <dbus/dbus-glib-lowlevel.h>
#include <clutter/clutter-main.h>
#include <assert.h>

static void  tick_cb (DBusGProxy *proxy, 
                      const guint tick,
                      MediaPlayer * pl  )
{
  pl->signal_tick.emit();
}

static void  playing_cb (DBusGProxy *proxy, 
                         gboolean playing,
                         MediaPlayer * pl )
{
  if(playing)
    pl->signal_playing.emit();
  else
    pl->signal_paused.emit();
}


void RBMediaPlayer::init ( int argc, char* argv[] ){
  
  _clip = new Clip();
  
  GError *error = NULL;
  DBusGConnection *bus;
  DBusGProxy *bus_proxy;
  guint start_service_reply;
  
  /* get dbus connection */
	bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
	if (error)
		assert(!"Couldn't create the dbus connection");
  
  /* if rhythmbox is not started, start it */
  bus_proxy = dbus_g_proxy_new_for_name (bus,
						       "org.freedesktop.DBus",
						       "/org/freedesktop/DBus",
						       "org.freedesktop.DBus");

	if (!dbus_g_proxy_call (bus_proxy, "StartServiceByName", &error,
					G_TYPE_STRING, "org.gnome.Rhythmbox",
					G_TYPE_UINT, 0,
					G_TYPE_INVALID,
					G_TYPE_UINT, &start_service_reply,
					G_TYPE_INVALID)) {
		g_warning ("%s", error->message);
		exit (1);
	}
  
  _shell = dbus_g_proxy_new_for_name_owner (bus,
					      "org.gnome.Rhythmbox",
					      "/org/gnome/Rhythmbox/Shell",
					      "org.gnome.Rhythmbox.Shell",
					      &error);
	if (error) {
		_shell = NULL;
		_player = NULL;
		_plman = NULL;
		assert(!"Couldn't create the shell proxy");
	}
  
	_player = dbus_g_proxy_new_from_proxy (_shell,
						     "org.gnome.Rhythmbox.Player",
						     "/org/gnome/Rhythmbox/Player");
	if (_player == NULL) {
		g_object_unref (G_OBJECT (_shell));
		_shell = NULL;
		_player = NULL;
		_plman = NULL;
		assert(!"Couldn't create the player proxy");
	}
	
	_plman = dbus_g_proxy_new_from_proxy (_shell,
						     "org.gnome.Rhythmbox.PlaylistManager",
						     "/org/gnome/Rhythmbox/PlaylistManager");
	if (_plman == NULL) {
		g_object_unref (G_OBJECT (_shell));
		_shell = NULL;
		_player = NULL;
		assert(!"Couldn't create the playlist manager proxy");
	}
	g_clear_error (&error);
  
  //Setup signals
  // http://www-128.ibm.com/developerworks/linux/library/l-dbus.html
  // http://dbus.freedesktop.org/doc/api/html/group__DBusGLib.html#gfac56b6025a90951510d33423ff04120
  dbus_g_proxy_add_signal (_player, "elapsedChanged", G_TYPE_UINT, G_TYPE_INVALID);
  dbus_g_proxy_connect_signal (_player, "elapsedChanged", G_CALLBACK(tick_cb), (void*)this, NULL);
  
  dbus_g_proxy_add_signal (_player, "playingChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
  dbus_g_proxy_connect_signal (_player, "playingChanged", G_CALLBACK(playing_cb), (void*)this, NULL);
  
  fillClip();
}

void RBMediaPlayer::play (  ){
  printf("RBMediaPlayer::play\n");
  GError *error = NULL;
  if(!isPlaying()){
    dbus_g_proxy_call (_player, "playPause", &error, G_TYPE_BOOLEAN, FALSE, 
        G_TYPE_INVALID, G_TYPE_INVALID);
    if (error) {
      g_warning ("%s", error->message);
  	}
  	fillClip();
	}
}

void RBMediaPlayer::pause (  ){
  printf("RBMediaPlayer::pause\n");
  GError *error = NULL;
  if(isPlaying()){
    dbus_g_proxy_call (_player, "playPause", &error, G_TYPE_BOOLEAN, FALSE, 
        G_TYPE_INVALID, G_TYPE_INVALID);
    if (error) {
      g_warning ("%s", error->message);
  	}
  	fillClip();
  }
}

void RBMediaPlayer::setURI ( std::string uri ){
  fillClip();
}

float RBMediaPlayer::getProgress (  ){
  printf("RBMediaPlayer::getProgress\n");
  GError *error = NULL;
  int elapsed = 0;
  if(isPlaying()){
    dbus_g_proxy_call (_player, "getElapsed", &error,  
        G_TYPE_INVALID, G_TYPE_UINT, &elapsed, G_TYPE_INVALID);
    if (error) {
      g_warning ("%s", error->message);
  	}
  }
  return elapsed/((float)getDuration());
}

void RBMediaPlayer::setProgress ( float progress ){
  printf("RBMediaPlayer::setProgress\n");
  GError *error = NULL;
  if(isPlaying()){
    dbus_g_proxy_call (_player, "setElapsed", &error,  
        G_TYPE_INVALID, G_TYPE_UINT, progress*getDuration(), G_TYPE_INVALID);
    if (error) {
      g_warning ("%s", error->message);
  	}
  }
}

bool RBMediaPlayer::isPlaying (  ){
  printf("RBMediaPlayer::isPlaying\n");
  GError *error = NULL;
  gboolean  playing;
  /* Check if not already playing */
  dbus_g_proxy_call (_player, "getPlaying", &error, G_TYPE_INVALID, G_TYPE_BOOLEAN, 
    &playing, G_TYPE_INVALID);
  if (error) {
  		g_warning ("%s", error->message);
  }  
  return playing;
   
}

int RBMediaPlayer::getDuration (  ){
  return _clip->getDuration();
}



float RBMediaPlayer::getVolume(  ){
  /*double volume ;
  GError *error = NULL;
  dbus_g_proxy_call (_player, "getVolume", &error, G_TYPE_INVALID, G_TYPE_DOUBLE, &volume, 
    G_TYPE_INVALID);
  if (error) {
  		assert(!"Couldn't get the player's volume");
  }  
  printf("VOlume get: %f\n", volume);*/
  return 1.0;
}

void RBMediaPlayer::setVolume ( float percent ){
  /** @todo Doesn't work, suspecting a bug in RB */
/*  if(percent <= 1.0 && percent >= 0){
    printf("NEW VOlume set: %f\n", percent);
    double test = percent*100;
    GError *error = NULL;
    dbus_g_proxy_call (_player, "setVolume", &error, G_TYPE_DOUBLE, &test, 
      G_TYPE_INVALID, G_TYPE_INVALID );
    if (error) {
        printf("%s\n", error->message);
    		assert(!"Couldn't set the player's volume");
    }  
  }*/
}
// Uncomment this to get the full list of values stored in Rhythmbox
//void keylist (gpointer key, gpointer value, gpointer user_data){
//  gchar * keytmp = (gchar*) key;
//  printf("key: %s \n", keytmp);
//}

void RBMediaPlayer::fillClip ( ){
  // http://gaim-rhythmbox.googlecode.com/svn/trunk/src/gaim-rhythmbox.c
  // http://developer.gnome.org/doc/tutorials/gnome-libs/glib.html
  printf("RBMediaPlayer::fillClip\n");
  GError *error = NULL;
  gchar * uri;
  GValue *value;
  GHashTable * properties = NULL;
  
  // Get current playing uri
  dbus_g_proxy_call (_player, "getPlayingUri", &error, G_TYPE_INVALID, G_TYPE_STRING, 
    &uri, G_TYPE_INVALID);
  if (error) {
    g_warning ("%s", error->message);
    return;
  }  
  dbus_g_proxy_call (_shell, "getSongProperties", &error, 
    G_TYPE_STRING, uri, G_TYPE_INVALID,  
    dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), 
    &properties, G_TYPE_INVALID);
  if (error) {
    g_warning ("%s", error->message);
    _clip->setCanSeek(false);
  }  
  if(properties == NULL)
    g_warning ("Couldn't retrieve current's song details: %s", error->message);
  else{
    _clip->setURI(uri);
  	
    value = (GValue *) g_hash_table_lookup(properties, "artist");
  	if (value != NULL && G_VALUE_HOLDS_STRING(value)) {
  		_clip->setArtist(g_value_get_string(value));
  	}
  	value = (GValue *) g_hash_table_lookup(properties, "album");
  	if (value != NULL && G_VALUE_HOLDS_STRING(value)) {
  		_clip->setAlbum(g_value_get_string(value));
  	}
  	value = (GValue *) g_hash_table_lookup(properties, "title");
  	if (value != NULL && G_VALUE_HOLDS_STRING(value)) {
  		_clip->setTitle(g_value_get_string(value));
  	}
  	value = (GValue *) g_hash_table_lookup(properties, "duration");
  	if (value != NULL && G_VALUE_HOLDS_UINT(value)) {
  		_clip->setDuration(g_value_get_uint(value));
  		printf("Duration: %d\n", _clip->getDuration());
  	}
  	// Uncomment this to get the full list of values stored in Rhythmbox
  	//g_hash_table_foreach(properties, keylist, NULL);

  	const gchar * home = g_get_home_dir();
  	gchar * path = g_strconcat(home, "/.gnome2/rhythmbox/covers/", 
  	  _clip->getArtist().c_str(), " - ", _clip->getAlbum().c_str(), ".jpg", NULL);
  	_clip->setImagePath(path);
  	printf("path: %s\n", path);
  }
//  g_hash_table_lookup(properties, "Duration");
//  int * duration = (int*) g_hash_table_lookup(properties, "Duration");
}


Clip * RBMediaPlayer::getClip (  ){ 
  fillClip();
  return _clip;
}
