/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef SETTINGS_H
#define SETTINGS_H

#include <string>
#include <gconf/gconf-client.h>

/** @brief Quick access to the instance */
#define SETTINGS Settings::getInstance() // To shorthen the syntax

/**
 * @class Settings
 * @brief Holds the settings of the application.
 *
 * This singleton object is used to retrieve the settings of the application.  
 * All Sofa settings should be under "/apps/sofa".  This path is not hard coded
 * in the methods to allow a module to read other application's settings as per 
 * requirements.
 *
 * There are three subsections under "/apps/sofa": 
 * <ul>
 * <li><b>theme</b>: settings saved under themes should concern only graphical 
 * settings such ascolor, font, size.</li>
 * <li><b>modules</b>: settings saved under modules should concern modules.  
 * Each module should have its own subsection.</li> 
 * <li><b>keybindings</b>: keep keybindings.</li>, , . 
 * </ul>
 * For example, to access the background_color, one should call 
 * SETTINGS->getString("/apps/sofa/theme/background_color").
 */
class Settings {
  // Associations
  // Attributes
  private: 
  GConfClient* _client;
  
  // Operations
  public:
    
    /**
     * @brief This class is a singleton.
     * @return A pointer to the Settings instance.
     */
    static Settings* getInstance (  );
    
    /**
     * @brief Initialise the connection to the settings database.
     * @param argc The argument count passed to main().
     * @param argv The argument vector passed to main().
     */
    void init ( int argc, char * argv[] );
    
    /**
     * @brief Return a bool setting.
     * @param setting The path to the key.
     */
    bool getBool (const gchar * setting );
    
    /**
     * @brief Return an integer setting.
     * @param setting The path to the key.
     */
    int getInt (const gchar * setting );
    
    /**
     * @brief Return a string setting.
     * @param setting The path to the key.
     */
    gchar * getString (const gchar * setting );
    
    
    gchar * getStringList (const gchar * setting, int pos );
    int getStringListSize (const gchar * setting );
    
    /**
     * @brief Sets a bool setting.
     * @param setting The path to the key.
     * @param value The value.
     */
    void setBool (const gchar * setting, bool value );
    
    /**
     * @brief Sets a int setting.
     * @param setting The path to the key.
     * @param value The value.
     */
    void setInt (const gchar * setting, int value );
    
    /**
     * @brief Sets a string setting.
     * @param setting The path to the key.
     * @param value The value.
     */
    void setString (const gchar * setting, const gchar * value );
    
     /**
     * @brief Sets a string list setting.
     * @param setting The path to the key.
     * @param value The value.
     * @param size the number of items in the list
     */
    gchar * setStringList (const gchar * setting, 
                                gchar * *value, 
                                int size );

};

#endif
