/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <config.h>

#include "clutterview.h"
#include "mediaplayer.h"
#include "cluttermediaplayer.h"
#include "../control.h"
#include "../clip.h"
#include "../settings.h"
#include "../menus/viewers/videomenu.h"
#include "../menus/selecters/selectionmenu.h"
#include "../menus/viewers/musicmenu.h"
#include "../debug.h"

#include <assert.h>
#include <clutter-cairo-0.1/clutter-cairo.h>
#include <sstream> // std::istringstream
 
#define TITLE_X 0.025
#define TITLE_Y 0.025

#ifndef MAX
#define MAX( x, y ) ((x) > (y) ? (x) : (y))
#endif


static ClutterView * VIEW;
/*
 * When the timeline is completed, this will destroy the actors
 */
void destroy_cb (ClutterTimeline *timeline, 
	                gpointer         data)
{
  clutter_group_remove ( CLUTTER_GROUP(VIEW->getStage()), (ClutterActor * ) data);
  clutter_actor_destroy( (ClutterActor * ) data);
}


/* 
 * Any processing required when key press occurs should be done here.
 */
static bool key_press_event (ClutterStage *stage,
                             ClutterEvent *event,
                             gpointer      data) {
  CMenu ** cmenu = (CMenu**) data;
                                   
  int key = clutter_key_event_symbol(&event->key);

  return Control::getInstance()->getView()->signal_keypressed(key);
}

/*
 * When the timeline is completed, this will start the next one connected
 */
void next_cb (ClutterTimeline *timeline, 
	                gpointer         data)
{
  // Next animation 
  ClutterTimeline * time = (ClutterTimeline * ) data;
  clutter_timeline_start (time);
}


ClutterView::ClutterView(){
  _currentMenu = NULL;
}

void ClutterView::init ( int argc, char * argv[] ){
  
  //Init the pointer used by the callbacks
  VIEW = this;
  
  clutter_init (&argc, &argv);
  _stage = clutter_stage_get_default ();
  g_object_set(_stage, "fullscreen", TRUE, NULL);
  //clutter_actor_set_size(_stage, 1024, 768);
  clutter_actor_get_size(_stage, &screenWidth, &screenHeight);
  
  g_signal_connect (G_OBJECT (_stage), "key_press_event",
			  G_CALLBACK (key_press_event), &_current);
			  
 
  clutter_color_parse(
    SETTINGS->getString("/apps/sofa/theme/selected_color"), 
    &selectedColor
    );
  clutter_color_parse(
    SETTINGS->getString("/apps/sofa/theme/menu_color"), 
    &menuColor
    );
    
  // Set fonts from Settings
  bigMenuFont = SETTINGS->getString("/apps/sofa/theme/menu_font");
  int space = bigMenuFont.find_last_of(" ");
  int size = 0;
  
  if( space == -1){
    //Invalid font entered, replace with default  
    log("Check /apps/sofa/theme/menu_font, it contains an invalid font description\n");
    bigMenuFont = "Sans Reguler 24";
    smallMenuFont = "Sans Regular 12";

    bigFontLineHeight = 24;
    smallFontLineHeight = 12;
  } else {
    std::istringstream sizestring(bigMenuFont.substr(space));
    
    smallMenuFont = bigMenuFont.substr(0, space + 1);
    bigMenuFont   = bigMenuFont.substr(0, space + 1);
    
    sizestring >> size;
    bigFontLineHeight = (int)(size/64.0*x(0.1));
    smallFontLineHeight = (int)(size/64.0*x(0.1)/2.0);
    
    std::ostringstream outsize;
    outsize << smallFontLineHeight;
    smallMenuFont.append(outsize.str());   
    
    std::ostringstream outsize2;
    outsize2 << bigFontLineHeight;
    bigMenuFont.append(outsize2.str());   
  }
  logoFont = "MgOpen Moderna Bold ";
  
  std::ostringstream outsize3;
  outsize3 << y(0.125);
  logoFont.append(outsize3.str());   
 
  
  
}

void ClutterView::update (  Menu::Updates up  ){
  log("ClutterView::update");
  if(!_currentMenu)
    return;
    
  Menu * menu = CONTROL->getSelectedMenu();
  if(!menu) 
    return; 
  
  SelectionMenu * selectmenu = dynamic_cast<SelectionMenu*>(menu);
  
  MusicMenu * musicmenu  = dynamic_cast<MusicMenu*>(menu);;
  
  VideoMenu * videomenu  = dynamic_cast<VideoMenu*>(menu);;
  
  switch(up){
    case Menu::SELECTED_MENU_CHANGED:
      log("Menu::SELECTED_MENU_CHANGED\n");
      goTo(menu);
      break;
    case Menu::HIGHLIGHTED_MENU_CHANGED:
      log("Menu::HIGHLIGHTED_MENU_CHANGED\n");
      if(selectmenu){
        updateSelectionMenu(selectmenu);       
      }
      break;
    case Menu::PROPERTIES_CHANGED:
      log("Menu::PROPERTIES_CHANGED\n");
      if(musicmenu){
        //Current song changed ==> Fade out current, fade in next
        _current->fade_out = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 90);
        g_signal_connect (
          _current->fade_out, 
          "completed", 
          G_CALLBACK (destroy_cb), 
          _current->menu_options); 
        ClutterAlpha * alpha = clutter_alpha_new_full (_current->fade_out,
                                      CLUTTER_ALPHA_RAMP_INC,
                                      NULL, NULL);
        ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0xff, 0x00); 
        clutter_behaviour_apply (o_behave,  _current->menu_options);
        
        _current->menu_options = buildViewerMenu(musicmenu);
        clutter_group_add(CLUTTER_GROUP (_stage), _current->menu_options);
        _current->fade_in = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 90);
        alpha = clutter_alpha_new_full (_current->fade_in,
                                    CLUTTER_ALPHA_RAMP_INC,
                                    NULL, NULL);
        o_behave  = clutter_behaviour_opacity_new (alpha, 0x00, 0xff); 
        clutter_behaviour_apply (o_behave,  _current->menu_options);
        
        g_signal_connect (_current->fade_out, "completed", G_CALLBACK (next_cb), _current->fade_in);  
        clutter_timeline_start (_current->fade_out);
      }
      break;
    case Menu::LOADING_CHANGED:
      log("Menu::LOADING_CHANGED\n");
      if(selectmenu){
        updateLoadingSelectionMenu(selectmenu);
      }
      break;
    case Menu::TICK:
      log("Menu::TICK\n");
      if(musicmenu){
        updateMusicMenu(musicmenu);
      } else if (videomenu ){
        updateVideoMenu(videomenu);
      }
      break;
    case Menu::CONTROLS_ON:
      log("Menu::CONTROLS_ON\n");
      if (videomenu ){
        showControls();
      }
      break;  
    case Menu::CONTROLS_OFF:
      log("Menu::CONTROLS_OFF\n");
      if (videomenu ){
        hideControls();
      }
      break;
  }

 
}

void ClutterView::show (  ){
  _current = buildLogo();
  
  _current->background = clutter_cairo_new (x(1.0), y(1.0));
  createBackground(_current->background);
  
  clutter_group_add(CLUTTER_GROUP (_stage), _current->background);
  clutter_group_add(CLUTTER_GROUP (_stage), _current->title);
  clutter_group_add(CLUTTER_GROUP (_stage), _current->subtitle);
  
  // buildSelectionMenu needs to be called here because if called from init() it's empty
  _currentMenu = CONTROL->getRootMenu();
  _current->menu_options = buildSelectionMenu(static_cast<SelectionMenu*>(_currentMenu));
  _currentPos = static_cast<ListMenu*>(_currentMenu)->getHighlightedPos();
  clutter_group_add (CLUTTER_GROUP (_stage), _current->menu_options);
  
  //Initial fade in of the menu options
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->fade_out,
                                  CLUTTER_ALPHA_RAMP_INC,
                                  NULL, NULL);
                                  
  ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0X00, 0xff);   
  clutter_behaviour_apply (o_behave,  _current->menu_options);
 
  // Start the show
  clutter_actor_show_all (CLUTTER_ACTOR (_stage));

  clutter_timeline_start(_current->fade_in);
  
  clutter_main();
}
/* ----------------------------------------------------------------------------------------*/

void ClutterView::close (  ){
  _current->fade_out = clutter_timeline_new (ClutterView::LOGO_DUR, 60);
  
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->fade_out,
                                  CLUTTER_ALPHA_RAMP_INC,
                                  NULL, NULL);
                                  
  ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0X50, 0x00); 
  ClutterBehaviour * o_behave2 = clutter_behaviour_opacity_new (alpha, 0Xff, 0x00); 
  
  g_signal_connect (_current->fade_out, "completed", G_CALLBACK (clutter_main_quit), NULL);
  
  clutter_behaviour_apply (o_behave,  _current->title);
  clutter_behaviour_apply (o_behave,  CLUTTER_ACTOR(_current->image));
  clutter_behaviour_apply (o_behave2, _current->menu_options);

  
  clutter_timeline_start (_current->fade_out);
  
}
/* ----------------------------------------------------------------------------------------*/

CMenu * ClutterView::buildLogo (){
  CMenu * tmp = g_new0(CMenu, 1);
  ClutterKnot knots[]  = {{ x(0.4), y(0.3) }, { x(0.4), y(0.4) }};
  ClutterKnot knots3[] = {{ x(0.4), y(0.4) }, { x(TITLE_X), y(TITLE_Y) }};
  ClutterKnot knots2[] = {{ x(0.4), y(0.8) }, { x(0.4), y(0.65) }};
  
  ClutterActor * actor; 
  ClutterAlpha * alpha;
  ClutterBehaviour * p_behave, * p_behave2, * o_behave, *o_behave2;
  ClutterActor * ret = clutter_group_new();
  ClutterColor colorMenu;
  
  clutter_color_parse(SETTINGS->getString("/apps/sofa/theme/menu_color"), &colorMenu);
  
  actor = clutter_label_new_with_text (logoFont.c_str(), "Sofa");
  clutter_label_set_color(CLUTTER_LABEL(actor), &colorMenu);
  clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
  tmp->title = actor;
  
  actor = clutter_label_new_with_text (logoFont.c_str(), "Sofa");
  clutter_actor_rotate_x (actor, 180, 0,0);
  clutter_label_set_color(CLUTTER_LABEL(actor), &colorMenu);
  clutter_actor_set_opacity(actor, 50 );
  tmp->subtitle = actor;
  
  //Fade in
  tmp->fade_in = clutter_timeline_new (ClutterView::LOGO_DUR, 60);
  
  alpha = clutter_alpha_new_full (tmp->fade_in,
                                  CLUTTER_ALPHA_RAMP_INC,
                                  NULL, NULL);
                                  
  p_behave  = clutter_behaviour_path_new (alpha, knots, 2);
  p_behave2 = clutter_behaviour_path_new (alpha, knots2, 2);  
  
  o_behave  = clutter_behaviour_opacity_new (alpha, 0X33, 0xff); 
  o_behave2 = clutter_behaviour_opacity_new (alpha, 0X11, 0x77); 
  
  clutter_behaviour_apply (p_behave,  tmp->title);
  clutter_behaviour_apply (p_behave2, tmp->subtitle);
  clutter_behaviour_apply (o_behave,  tmp->title);
  clutter_behaviour_apply (o_behave2, tmp->subtitle);
  
  //Fade out 
  ClutterTimeline * t1 = clutter_timeline_new (ClutterView::LOGO_DUR, 60);
  
  alpha = clutter_alpha_new_full (t1,
                                  CLUTTER_ALPHA_RAMP_DEC,
                                  NULL, NULL);
                                  
  o_behave  = clutter_behaviour_opacity_new (alpha, 0X50, 0xff); 
  o_behave2 = clutter_behaviour_opacity_new (alpha, 0X00, 0x77); 
  
  clutter_behaviour_apply (o_behave,  tmp->title);
  clutter_behaviour_apply (o_behave2, tmp->subtitle);
  
  //Move out 
  tmp->fade_out = clutter_timeline_new (ClutterView::LOGO_DUR/2, 60);
  
  alpha = clutter_alpha_new_full (tmp->fade_out,
                                  CLUTTER_ALPHA_RAMP_INC,
                                  NULL, NULL);
                                  
  p_behave  = clutter_behaviour_path_new (alpha, knots3, 2);
  p_behave2  = clutter_behaviour_scale_new (alpha, 1.0, 0.45, CLUTTER_GRAVITY_NONE);
  
  clutter_behaviour_apply (p_behave,  tmp->title);
  clutter_behaviour_apply (p_behave2,  tmp->title);
  
  g_signal_connect (tmp->fade_in, "completed", G_CALLBACK (next_cb), t1);  
  g_signal_connect (t1, "completed", G_CALLBACK (next_cb), tmp->fade_out);

  return tmp;
}
/* ----------------------------------------------------------------------------------------*/

ClutterActor * ClutterView::buildSelectionMenu (SelectionMenu * menu){
  ClutterActor * actor; 
  ClutterActor * group;
  ClutterActor * ret = clutter_group_new();
  
  int pos = menu->getHighlightedPos();
  _currentPos = pos;
  // scrolls the menu to the correct position 
  int height = - pos*getScrollLineHeight(); 
  
  if(menu){
    
    for(int j = 0; j <= menu->size(); j++){
      Menu * sub = menu->get(j);
      if(sub){
        group = clutter_group_new();
        
        GdkPixbuf * pixbuf = gdk_pixbuf_new_from_file_at_size( 
            sub->getImagePath(), 
            bigFontLineHeight,
            bigFontLineHeight, 
            NULL);
        if (pixbuf){
          actor = clutter_texture_new_from_pixbuf (pixbuf);
          clutter_actor_set_position(actor, -bigFontLineHeight, height+y(0.01));
          clutter_actor_set_size(actor, bigFontLineHeight, bigFontLineHeight);
          clutter_group_add (CLUTTER_GROUP (group), actor);
        } else { // Place holder for the image
          actor = clutter_label_new_with_text (smallMenuFont.c_str(), "sofa");
          clutter_actor_set_position(actor, -bigFontLineHeight, height+y(0.01));
          clutter_actor_set_size(actor, bigFontLineHeight, bigFontLineHeight);
          clutter_group_add (CLUTTER_GROUP (group), actor);
        }
        
        actor = clutter_label_new_with_text (bigMenuFont.c_str(), sub->getTitle());
        if(sub == menu->getHighlighted()){
          clutter_label_set_color(CLUTTER_LABEL(actor), &selectedColor);
        }else{
          clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
        }
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        clutter_actor_set_position(actor, 0, height);
        clutter_actor_set_size(CLUTTER_ACTOR(actor), 0, bigFontLineHeight);
        clutter_group_add (CLUTTER_GROUP (group), actor);
        height += (int)(1.1*bigFontLineHeight);
        
        actor = clutter_label_new_with_text (smallMenuFont.c_str(), sub->getSubtitle());
        if(sub == menu->getHighlighted()){
          clutter_label_set_color(CLUTTER_LABEL(actor), &selectedColor);
        }else{
          clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
        }
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        clutter_actor_set_position(actor, 0, height);
        clutter_actor_set_size(CLUTTER_ACTOR(actor), 0, smallFontLineHeight);
        clutter_group_add (CLUTTER_GROUP (group), actor);
        
        clutter_group_add (CLUTTER_GROUP (ret), group);
        height += smallFontLineHeight + LINE_SPACING;
      }
    }  
  
  }
  clutter_actor_set_opacity(ret, 0 );
  clutter_actor_set_position(ret, x(0.5), y(0.5));
  clutter_actor_show_all (CLUTTER_ACTOR (ret));
  
  return ret;
}

void ClutterView::updateSelectionMenu(SelectionMenu * selmenu){
  int xx; 
  int yy; 
  
  ClutterActor * menu = _current->menu_options;
  int pos = selmenu->getHighlightedPos();
  
  for(int j = 0; j <= selmenu->size(); j++){
    Menu * sub = selmenu->get(j);
    if(sub){
      ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(menu), j);
      
  	      
      if(sub == selmenu->getHighlighted()){
        
        ClutterActor * actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 1);
        clutter_label_set_color(CLUTTER_LABEL(actor), &selectedColor);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        
        actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 2);
        clutter_label_set_color(CLUTTER_LABEL(actor), &selectedColor);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
          
      } else {
      
        ClutterActor * actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 1);
        clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        
        actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 2);
        clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
      }   
  
    }
  }
  
  // Animate the scroll
  clutter_actor_get_abs_position(_current->menu_options, &xx, &yy);
  ClutterKnot knots[]  = {{ x(0.5), yy }, 
                          { x(0.5), (_currentPos < pos ? 
                              yy - getScrollLineHeight(): 
                              yy + getScrollLineHeight()) }};
  _current->slide = clutter_timeline_new ((ClutterView::SEL_CHANGE_DUR), 60);
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->slide,
                            CLUTTER_ALPHA_RAMP_INC,
                            NULL, NULL);
                                  
  ClutterBehaviour * p_behave  = clutter_behaviour_path_new (alpha, knots, 2);   
  clutter_behaviour_apply (p_behave,  _current->menu_options);

  clutter_timeline_start (_current->slide);
  
  _currentPos = pos;
}

void ClutterView::updateLoadingSelectionMenu(SelectionMenu * selmenu){
  int xx = -bigFontLineHeight; 
  int pos = selmenu->getHighlightedPos();
  // scrolls the menu to the correct position 
  int yy = pos*getScrollLineHeight(); 
 
  
  ClutterActor * menu = _current->menu_options;
  
  
  for(int j = 0; j <= selmenu->size(); j++){
    Menu * sub = selmenu->get(j);
    if(sub){
      ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(menu), j);
      
      if(sub == selmenu->getHighlighted()){
        if(sub->getPercentLoaded() > 0){// && !clutter_timeline_is_playing(_current->loading)){ //If the submenu is loading
          ClutterActor * image = clutter_group_get_nth_child(CLUTTER_GROUP(group), 0);
          
          ClutterKnot knots[]  = {{ xx , yy }, {xx, yy-50 }};
          
          _current->loading = clutter_timeline_new(MENU_CHANGE_DUR, 60);
          g_object_set (_current->loading, "loop", TRUE, 0);  
          ClutterAlpha * alpha = clutter_alpha_new_full (_current->loading,
                            CLUTTER_ALPHA_SINE,
                            NULL, NULL);
          ClutterBehaviour * p_behave  = clutter_behaviour_path_new (alpha, knots, 2);   
          clutter_behaviour_apply (p_behave, image );
          clutter_timeline_start (_current->loading);
        }
        
      } 
  
    }
  }
 
}

int ClutterView::x(float percent){
  return (int)(percent*screenWidth);
}

int ClutterView::y(float percent){
  return (int)(percent*screenHeight);
}

int ClutterView::getScrollLineHeight(){
  return smallFontLineHeight + (int)(1.3*bigFontLineHeight) + LINE_SPACING;
}

ClutterActor *  ClutterView::buildViewerMenu(Menu* menu){
  MusicMenu * music = dynamic_cast<MusicMenu*>(menu);
  if(music)
    return buildMusicMenu(music);
  
  VideoMenu * video = dynamic_cast<VideoMenu*>(menu);
  if(video)
    return buildVideoMenu(video);
    
    
  return NULL;
}

ClutterActor *  ClutterView::buildMusicMenu(MusicMenu* menu){
  Clip * clip = menu->getCurrentClip();
  
  ClutterActor * actor; 
  ClutterActor * group;
  ClutterActor * ret = clutter_group_new();
  GError * error = NULL;
  // scrolls the menu to the correct position 
  int height = y(0.40) ; 
  
  if(menu){
    group = clutter_group_new();
        
      GdkPixbuf * pixbuf = gdk_pixbuf_new_from_file_at_size( 
          clip->getImagePath().c_str(), 
          x(0.35), 
          x(0.35), 
          &error);
      if (pixbuf){
        actor = clutter_texture_new_from_pixbuf (pixbuf);
        clutter_actor_set_position(actor, x(0.10), (int)(y(0.5)- x(0.35)/2.0));
        clutter_actor_set_size(actor, x(0.35), x(0.35));
        clutter_group_add (CLUTTER_GROUP (group), actor);
      } else {
        g_warning("%s", error->message);
        GdkPixbuf * pixbuf = gdk_pixbuf_new_from_file_at_size( 
          menu->getImagePath(), 
          x(0.35), 
          x(0.35), 
          NULL);
        if (pixbuf){
          actor = clutter_texture_new_from_pixbuf (pixbuf);
          clutter_actor_set_position(actor, x(0.10),(int)(y(0.5)- x(0.35)/2.0));
          clutter_actor_set_size(actor, x(0.35), x(0.35));
          clutter_group_add (CLUTTER_GROUP (group), actor);
        } else { // Place holder for the image
          actor = clutter_label_new_with_text (smallMenuFont.c_str(), "sofa");
          clutter_actor_set_position(actor, -bigFontLineHeight, height+y(0.01));
          clutter_actor_set_size(actor, bigFontLineHeight, bigFontLineHeight);
          clutter_group_add (CLUTTER_GROUP (group), actor);
        }
      }
      
      actor = clutter_label_new_with_text (bigMenuFont.c_str(), clip->getTitle().c_str());
      clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
      clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
      clutter_actor_set_position(actor, x(0.50), height);
      clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.50), bigFontLineHeight);
      clutter_group_add (CLUTTER_GROUP (group), actor);
      height += (int)(1.1*bigFontLineHeight);
      
      actor = clutter_label_new_with_text (bigMenuFont.c_str(), clip->getAlbum().c_str());
      clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
      clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
      clutter_actor_set_position(actor, x(0.50), height);
      clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.50), bigFontLineHeight);
      clutter_group_add (CLUTTER_GROUP (group), actor);
      height += (int)(1.1*bigFontLineHeight);
      
      actor = clutter_label_new_with_text (bigMenuFont.c_str(), clip->getArtist().c_str());
      clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
      clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
      clutter_actor_set_position(actor, x(0.50), height);
      clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.50), bigFontLineHeight);
      clutter_group_add (CLUTTER_GROUP (group), actor);
      height += (int)(2.1*bigFontLineHeight);
      
      //Progress bar
      if(clip->canSeek()){
        actor = clutter_rectangle_new_with_color ( &menuColor);
        clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.40), y(0.025));
        clutter_actor_set_position(actor, x(0.5), height);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        clutter_group_add (CLUTTER_GROUP (group), actor);
        
        double progress = menu->getMediaPlayer()->getProgress();
        actor = clutter_rectangle_new_with_color ( &selectedColor);
        clutter_actor_set_size(CLUTTER_ACTOR(actor), (int)(x(0.40)*progress), y(0.025));
        clutter_actor_set_position(actor, x(0.5), height);
        clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
        clutter_group_add (CLUTTER_GROUP (group), actor);
      }
      clutter_group_add (CLUTTER_GROUP (ret), group);
       
  }
  clutter_actor_set_opacity(ret, 0 );
  clutter_actor_show_all (CLUTTER_ACTOR (ret));
  
  return ret;
}

void ClutterView::updateMusicMenu(MusicMenu * menu){
  double progress = menu->getMediaPlayer()->getProgress();
  ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(_current->menu_options), 0);
  ClutterActor * actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 5);
  clutter_actor_set_size(CLUTTER_ACTOR(actor),(int) (x(0.40)*progress), y(0.025));
      
}

void ClutterView::updateVideoMenu(VideoMenu * menu){
  double progress = menu->getMediaPlayer()->getProgress();
  ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(_current->menu_options), 1);
  ClutterActor * actor = clutter_group_get_nth_child(CLUTTER_GROUP(group), 2);
  clutter_actor_set_size(CLUTTER_ACTOR(actor),(int)( x(0.80)*progress), y(0.025));
  
}


ClutterActor *  ClutterView::buildVideoMenu(VideoMenu* menu){
  ClutterActor * actor; 
  ClutterActor * group;
  ClutterActor * ret = clutter_group_new();
  
  if(menu){
    ClutterMediaPlayer * cmp = static_cast<ClutterMediaPlayer *> (menu->getMediaPlayer());
    if(cmp){
      actor = cmp->getClutterMediaTexture();
      g_object_set (G_OBJECT(actor), "sync-size", FALSE, NULL);
      clutter_actor_set_position(actor, 0, 0);
      clutter_actor_set_size(actor, x(1), y(1));
      clutter_group_add (CLUTTER_GROUP (ret), actor);
    }
      
    group = clutter_group_new();
        
    actor = clutter_label_new_with_text (smallMenuFont.c_str(), menu->getTitle());
    clutter_label_set_color(CLUTTER_LABEL(actor), &menuColor);
    clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 255);
    clutter_actor_set_position(actor, x(0.1), y(0.85));
    clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.5),  y(0.1));
    clutter_group_add (CLUTTER_GROUP (group), actor);
    
    //Progress bar
    actor = clutter_rectangle_new_with_color ( &menuColor);
    clutter_actor_set_size(CLUTTER_ACTOR(actor), x(0.8), y(0.025));
    clutter_actor_set_position(actor, x(0.1), y(0.9));
    clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 200);
    clutter_group_add (CLUTTER_GROUP (group), actor);
    
    double progress = menu->getMediaPlayer()->getProgress();
    actor = clutter_rectangle_new_with_color ( &selectedColor);
    clutter_actor_set_size(CLUTTER_ACTOR(actor),(int)( x(0.8)*progress), y(0.025));
    clutter_actor_set_position(actor, x(0.1),  y(0.9));
    clutter_actor_set_opacity(CLUTTER_ACTOR(actor), 200);
    clutter_group_add (CLUTTER_GROUP (group), actor);   
      
    _current->controls = group;
    
    clutter_group_add (CLUTTER_GROUP (ret), group);
    
  }
  clutter_actor_set_opacity(ret, 0 );
  clutter_actor_show_all (CLUTTER_ACTOR (ret));
  
  return ret;
}

void  ClutterView::showControls(){
  ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(_current->menu_options), 1);
  
  _current->controls_tl = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 60);
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->controls_tl,
                                CLUTTER_ALPHA_RAMP_INC,
                                NULL, NULL);
    
  ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0, 0xF0); 
  clutter_behaviour_apply (o_behave,  group);
  clutter_timeline_start(_current->controls_tl);
}

void  ClutterView::hideControls(){
  ClutterActor * group = clutter_group_get_nth_child(CLUTTER_GROUP(_current->menu_options), 1);
  
  _current->controls_tl = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 60);
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->controls_tl,
                                CLUTTER_ALPHA_RAMP_INC,
                                NULL, NULL);
    
  ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0xF0, 0); 
  clutter_behaviour_apply (o_behave,  group);
  clutter_timeline_start(_current->controls_tl);
}


ClutterActor * ClutterView::getStage(){
  return _stage;
}

void ClutterView::goTo(Menu * tmp){
  //log("Change in menu detected, next menu: %s\n", tmp->getTitle());
  if (!tmp)
    return ;
    
  SelectionMenu * selectmenu = dynamic_cast<SelectionMenu*>(tmp);
  MusicMenu *     musicmenu  = dynamic_cast<MusicMenu*>(tmp);
  VideoMenu *     videomenu  = dynamic_cast<VideoMenu*>(tmp);
  
  ClutterActor * next = NULL;
  
  ClutterActor * newtitle = clutter_group_get_nth_child(
    CLUTTER_GROUP(_current->menu_options), 
    _currentPos
    );
  
  if(selectmenu){ // If it is a selection menu
    log("Next menu is a SelectionMEnu\n");
    _currentPos = selectmenu->getHighlightedPos();
    next = buildSelectionMenu(selectmenu);
  } else if(musicmenu){
    log("Next menu is a musicmenu\n");
    _currentPos = -1;
    next = buildViewerMenu(tmp);
  } else if(videomenu){
    log("Next menu is a VIDEO\n");
    _currentPos = -1;
    next = buildViewerMenu(tmp);
  } 
  clutter_group_add(CLUTTER_GROUP(_stage), next);
  //Remove the menu option to prevent its fade out
  //clutter_actor_set_position(newtitle, 10, 10);
  //clutter_actor_set_opacity(newtitle, 255);
  //clutter_group_show_all(CLUTTER_GROUP(newtitle));
  //clutter_group_add(CLUTTER_GROUP(_stage), newtitle);
  //clutter_group_remove(CLUTTER_GROUP(_current->menu_options), newtitle);
      
  /*int xx, yy;
  clutter_actor_get_abs_position(newtitle, &xx, &yy);
  
  ClutterKnot knots[]  = {{ 0, 0 }, 
                          { -(xx - x(TITLE_X)), -(yy -y(TITLE_Y)) }};
  */
  //Stop any loading animation
  clutter_timeline_stop(_current->loading);
  clutter_timeline_rewind (_current->loading);
  //Fade out
  _current->fade_out = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 60);
  g_signal_connect (_current->fade_out, "completed", G_CALLBACK (destroy_cb), _current->menu_options);  
  ClutterAlpha * alpha = clutter_alpha_new_full (_current->fade_out,
                                CLUTTER_ALPHA_RAMP_INC,
                                NULL, NULL);
                                      
  //ClutterBehaviour * p_behave  = clutter_behaviour_path_new (alpha, knots, 2);   
  //clutter_behaviour_apply (p_behave,  newtitle);

  ClutterBehaviour * o_behave  = clutter_behaviour_opacity_new (alpha, 0Xff, 0x00); 
  clutter_behaviour_apply (o_behave,  _current->menu_options);
  
  //Fade in
  _current->menu_options = next;    
  _current->fade_in = clutter_timeline_new ((ClutterView::MENU_CHANGE_DUR), 60);
  alpha = clutter_alpha_new_full (_current->fade_in,
                                CLUTTER_ALPHA_RAMP_INC,
                                NULL, NULL);
                                      
  //ClutterBehaviour * p_behave  = clutter_behaviour_path_new (alpha, knots, 2);   
  //clutter_behaviour_apply (p_behave,  newtitle);

  o_behave  = clutter_behaviour_opacity_new (alpha, 0x00, 0xff); 
  clutter_behaviour_apply (o_behave,  _current->menu_options);
  
  g_signal_connect (_current->fade_out, "completed", G_CALLBACK (next_cb), _current->fade_in);  
    
  clutter_timeline_start (_current->fade_out);
  
  /*clutter_actor_show_all (CLUTTER_ACTOR (next));
  clutter_group_add (CLUTTER_GROUP (_stage), next);
  
  _enterMenuTL = clutter_timeline_new (ClutterView::MENU_CHANGE_DUR, 60);
  g_signal_connect (_enterMenuTL, "new-frame", G_CALLBACK (enter_menu_cb), next);
    
  _leaveMenuTL = clutter_timeline_new (ClutterView::MENU_CHANGE_DUR, 60);
  g_signal_connect (_leaveMenuTL, "new-frame", G_CALLBACK (leave_menu_cb), _menu);
  g_signal_connect (_leaveMenuTL, "completed", G_CALLBACK (next_cb), _enterMenuTL);
  
  clutter_timeline_rewind (_leaveMenuTL);
  clutter_timeline_start (_leaveMenuTL);
  
  _menu = next;
  */

}

void ClutterView::createBackground (ClutterActor * bg)
{
  cairo_t         * cr;
  cairo_pattern_t * pat;
  
	cr = clutter_cairo_create(CLUTTER_CAIRO (bg));
	
  // Clear the background
	cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
	cairo_paint(cr);

	cairo_set_operator (cr, CAIRO_OPERATOR_OVER);

  cairo_scale(cr, x(1.0), y(1.0));
	
	// Set background color from Settings
  ClutterColor color;
  ClutterColor color2;
  
  clutter_color_parse(SETTINGS->getString("/apps/sofa/theme/background_color"), &color);
  if(CLUTTER_MAJOR_VERSION == 0 &&
     CLUTTER_MINOR_VERSION == 2 &&
     CLUTTER_MICRO_VERSION <= 3){
     // clutter_color_shade doesn't return the correct hue prior to 0.2.4
     color2.red   = MAX(color.red - 100,0);
     color2.green = MAX(color.green - 100,0);
     color2.blue  = MAX(color.blue - 100,0);
  } else //this should have been fixed in 0.2.4 or later
    clutter_color_shade(&color, &color2, 0.5);
  
	// Create a gradient pattern 
	pat = cairo_pattern_create_linear (0, 0, 0, 1);
	cairo_pattern_add_color_stop_rgb (pat, 0.0,  color2.red/255.0, color2.green/255.0, color2.blue/255.0);
	cairo_pattern_add_color_stop_rgb (pat, 0.25, color.red/255.0,  color.green/255.0,  color.blue/255.0);
  cairo_pattern_add_color_stop_rgb (pat, 0.75, color.red/255.0,  color.green/255.0,  color.blue/255.0);
	cairo_pattern_add_color_stop_rgb (pat, 1.0,  color2.red/255.0, color2.green/255.0, color2.blue/255.0);
	cairo_set_source (cr, pat);
	
	/* create rectangle & fill with the pattern */
	cairo_rectangle (cr, 0, 0, 1, 1);
	cairo_fill (cr);
	cairo_pattern_destroy (pat);

 	cairo_destroy(cr);
}
