/* 
 * Copyright (C) 2007 Pierre-Luc Beaudoin
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#ifndef MEDIAPLAYER_H
#define MEDIAPLAYER_H

#include <sigc++/trackable.h>
#include <sigc++/signal.h>

/**
 * @class MediaPlayer
 * @brief The base class for all media players.
 *
 * In the Component design pattern, this is the base to all menu types.
 * This includes menus without sub-menus like the music player.
 */

class MediaPlayer {
  // Associations
  // Attributes
  public:
     /** @brief Emited when the player goes into playing state. */ 
     sigc::signal<void>  signal_playing;
     
     /** @brief Emited when the player goes into paused state. */ 
     sigc::signal<void>  signal_paused;
     
     /** @brief Emited when the player goes into loading state. */ 
     sigc::signal<void>  signal_loading;
     
     /** @brief Emited when the progress has changed */ 
     sigc::signal<void>  signal_tick;
     
  // Operations
  public:
    /**
     * @brief Initialise the media player.
     * @param argc The number of command line parameters.
     * @param argv The command line parameters.
     */
    virtual void init ( int argc, char* argv[] ) = 0;
    
    /**
     * @brief Plays the current loaded URI.
     */
    virtual void play (  ) = 0;
    
    /**
     * @brief Pauses the current loaded URI.
     */
    virtual void pause (  ) = 0;
    
    /**
     * @brief Sets the URI to be played.
     * @param uri URI of the media to play.
     */
    virtual void setURI ( std::string uri ) = 0;
    
    /**
     * @brief Returns the position in the current media.
     * @return In percent, the position in the media.
     */
    virtual float getProgress (  ) = 0;
    
    /**
     * @brief Sets the position in the current media.
     * @param progress In percent, the position in the media.
     */
    virtual void setProgress ( float progress ) = 0;
    
    /**
     * @brief Returns the volume.
     * @returns In percent, the volume.
     */
    virtual float getVolume(  ) = 0;
    
    /**
     * @brief Sets the volume.
     * @param percent In percent, the new volume.
     */
    virtual void setVolume ( float percent ) = 0;
  
    /**
     * @brief Returns wheter the media player is playing or not.
     * @return True if the player is playing, False, otherwise.
     */
    virtual bool isPlaying (  ) = 0;
   
   /**
     * @brief Returns the duration of the media.
     * @return The duration of the media in seconds
     */
    virtual int getDuration (  ) = 0;
};

#endif
